#include <avr/io.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <util/delay.h>
//#include <avr/eeprom.h>
//#include <compat/ina90.h>
#include "GFX_LCD_Functions.h"

#define CMD 0		//A "0" tells the display: now a Command is following
#define PARAM 1		//A "1" tells the display: now a Parameter is following

const char DataInit[20] = {0x00,0x02,0x04,0x06,0x09,0x0B,0x0D,0x0F,0x00,0x02,0x04,0x06,0x09,0x0B,0x0D,0x0F,0x00,0x04,0x0B,0x0F};

// space // 32  - 43 // ! // '' // # // $ // % // & // ' // ( // ) // * // +
const char cASCII1[] PROGMEM = {	0x00 , 0x00 , 0x00 , 0x00 , 0x00,       
						0x00 , 0x06 , 0x5F , 0x06 , 0x00,       
						0x07 , 0x03 , 0x00 , 0x07 , 0x03,       
						0x24 , 0x7E , 0x24 , 0x7E , 0x24,       
						0x24 , 0x2B , 0x6A , 0x12 , 0x00,       
						0x63 , 0x13 , 0x08 , 0x64 , 0x63,       
						0x36 , 0x49 , 0x56 , 0x20 , 0x50,       
						0x00 , 0x07 , 0x03 , 0x00 , 0x00,       
						0x00 , 0x3E , 0x41 , 0x00 , 0x00,       
						0x00 , 0x41 , 0x3E , 0x00 , 0x00,       
						0x08 , 0x3E , 0x1C , 0x3E , 0x08,       
						0x08 , 0x08 , 0x3E , 0x08 , 0x08};

// , // 44 - 55 // -  // . // / // 0 // 1 // 2 // 3 // 4 // 5 // 6 // 7
const char cASCII2[] PROGMEM = {	0x00 , 0xE0 , 0x60 , 0x00 , 0x00,       
						0x08 , 0x08 , 0x08 , 0x08 , 0x08,       
						0x00 , 0x60 , 0x60 , 0x00 , 0x00,       
						0x20 , 0x10 , 0x08 , 0x04 , 0x02,       
						0x3E , 0x51 , 0x49 , 0x45 , 0x3E,       
						0x00 , 0x42 , 0x7F , 0x40 , 0x00,       
						0x62 , 0x51 , 0x49 , 0x49 , 0x46,       
						0x22 , 0x49 , 0x49 , 0x49 , 0x36,       
						0x18 , 0x14 , 0x12 , 0x7F , 0x10,       
						0x2F , 0x49 , 0x49 , 0x49 , 0x31,       
						0x3C , 0x4A , 0x49 , 0x49 , 0x30,       
						0x01 , 0x71 , 0x09 , 0x05 , 0x03};

// 8 // 56 - 67 // 9 // : // ; // < // = // > // ? // @ // A // B // C
const char cASCII3[] PROGMEM = {	0x36 , 0x49 , 0x49 , 0x49 , 0x36,       
						0x06 , 0x49 , 0x49 , 0x29 , 0x1E,       
						0x00 , 0x6C , 0x6C , 0x00 , 0x00,       
						0x00 , 0xEC , 0x6C , 0x00 , 0x00,       
						0x08 , 0x14 , 0x22 , 0x41 , 0x00,       
						0x24 , 0x24 , 0x24 , 0x24 , 0x24,       
						0x00 , 0x41 , 0x22 , 0x14 , 0x08,       
						0x02 , 0x01 , 0x59 , 0x09 , 0x06,       
						0x3E , 0x41 , 0x5D , 0x55 , 0x1E,       
						0x7E , 0x09 , 0x09 , 0x09 , 0x7E,       
						0x7F , 0x49 , 0x49 , 0x49 , 0x36,       
						0x3E , 0x41 , 0x41 , 0x41 , 0x22};

// D // 68 - 79 // E // F // G // H // I // J // K // L // M // N // O
const char cASCII4[] PROGMEM = {	0x7F , 0x41 , 0x41 , 0x41 , 0x3E,       
						0x7F , 0x49 , 0x49 , 0x49 , 0x41,       
						0x7F , 0x09 , 0x09 , 0x09 , 0x01,       
						0x3E , 0x41 , 0x49 , 0x49 , 0x7A,       
						0x7F , 0x08 , 0x08 , 0x08 , 0x7F,       
						0x00 , 0x41 , 0x7F , 0x41 , 0x00,       
						0x30 , 0x40 , 0x40 , 0x40 , 0x3F,       
						0x7F , 0x08 , 0x14 , 0x22 , 0x41,       
						0x7F , 0x40 , 0x40 , 0x40 , 0x40,       
						0x7F , 0x02 , 0x04 , 0x02 , 0x7F,       
						0x7F , 0x02 , 0x04 , 0x08 , 0x7F,       
						0x3E , 0x41 , 0x41 , 0x41 , 0x3E};

// P // 80 - 91 // Q // R // S // T // U // V // W // X // Y // Z // [
const char cASCII5[] PROGMEM = {	0x7F , 0x09 , 0x09 , 0x09 , 0x06,       
						0x3E , 0x41 , 0x51 , 0x21 , 0x5E,       
						0x7F , 0x09 , 0x09 , 0x19 , 0x66,       
						0x26 , 0x49 , 0x49 , 0x49 , 0x32,       
						0x01 , 0x01 , 0x7F , 0x01 , 0x01,       
						0x3F , 0x40 , 0x40 , 0x40 , 0x3F,       
						0x1F , 0x20 , 0x40 , 0x20 , 0x1F,       
						0x3F , 0x40 , 0x3C , 0x40 , 0x3F,       
						0x63 , 0x14 , 0x08 , 0x14 , 0x63,       
						0x07 , 0x08 , 0x70 , 0x08 , 0x07,       
						0x71 , 0x49 , 0x45 , 0x43 , 0x00,       
						0x00 , 0x7F , 0x41 , 0x41 , 0x00};

// \ // 92 - 103 // ] // ^ // _ // ` // a // b // c // d // e // f // g
const char cASCII6[] PROGMEM = {	0x02 , 0x04 , 0x08 , 0x10 , 0x20,       
						0x00 , 0x41 , 0x41 , 0x7F , 0x00,       
						0x04 , 0x02 , 0x01 , 0x02 , 0x04,       
						0x80 , 0x80 , 0x80 , 0x80 , 0x80,       
						0x00 , 0x03 , 0x07 , 0x00 , 0x00,       
						0x20 , 0x54 , 0x54 , 0x54 , 0x78,       
						0x7F , 0x44 , 0x44 , 0x44 , 0x38,       
						0x38 , 0x44 , 0x44 , 0x44 , 0x28,       
						0x38 , 0x44 , 0x44 , 0x44 , 0x7F,       
						0x38 , 0x54 , 0x54 , 0x54 , 0x18,       
						0x08 , 0x7E , 0x09 , 0x09 , 0x00,       
						0x18 , 0xA4 , 0xA4 , 0xA4 , 0x7C};

// h // 104 - 115 // i // j // k // l // m // n // o // p // q // r // s
const char cASCII7[] PROGMEM = {	0x7F , 0x04 , 0x04 , 0x78 , 0x00,       
						0x00 , 0x00 , 0x7D , 0x00 , 0x00,       
						0x40 , 0x80 , 0x84 , 0x7D , 0x00,       
						0x7F , 0x10 , 0x28 , 0x44 , 0x00,       
						0x00 , 0x00 , 0x7F , 0x40 , 0x00,       
						0x7C , 0x04 , 0x18 , 0x04 , 0x78,       
						0x7C , 0x04 , 0x04 , 0x78 , 0x00,       
						0x38 , 0x44 , 0x44 , 0x44 , 0x38,       
						0xFC , 0x44 , 0x44 , 0x44 , 0x38,       
						0x38 , 0x44 , 0x44 , 0x44 , 0xFC,       
						0x44 , 0x78 , 0x44 , 0x04 , 0x08,       
						0x08 , 0x54 , 0x54 , 0x54 , 0x20};

// t // 116 - 126 // u // v // w // x // y // z // { // | // } // ~
const char cASCII8[] PROGMEM = {	0x04 , 0x3E , 0x44 , 0x24 , 0x00,       
						0x3C , 0x40 , 0x20 , 0x7C , 0x00,       
						0x1C , 0x20 , 0x40 , 0x20 , 0x1C,       
						0x3C , 0x60 , 0x30 , 0x60 , 0x3C,       
						0x6C , 0x10 , 0x10 , 0x6C , 0x00,       
						0x9C , 0xA0 , 0x60 , 0x3C , 0x00,       
						0x64 , 0x54 , 0x54 , 0x4C , 0x00,       
						0x08 , 0x3E , 0x41 , 0x41 , 0x00,       
						0x00 , 0x00 , 0x7F , 0x00 , 0x00,       
						0x00 , 0x41 , 0x41 , 0x3E , 0x08,       
						0x02 , 0x01 , 0x02 , 0x01 , 0x00};      

/*
//------------------------------------------------------------------------------
//Open output Lcd_windows with coordinatex x1,y1 and x2,y2
//------------------------------------------------------------------------------
void Lcd_window (char Lx1, char Ly1, char Lx2, char Ly2)
{
	Lcd_sendbyte(CMD, 0x2A);		//Column adress
	Lcd_sendbyte(PARAM, Lx1);		//X-Position upper left corner
	Lcd_sendbyte(PARAM, Lx2);		//y-Position upper left corner
	Lcd_sendbyte(CMD, 0x2B);		//Page adressing
	Lcd_sendbyte(PARAM, Ly1);		//x-Position of lower right corner
	Lcd_sendbyte(PARAM, Ly2);		//y-Position of lower right corner
	Lcd_sendbyte(CMD, 0x2C);
}


//------------------------------------------------------------------------------
//Defines window area and fills with colour
//------------------------------------------------------------------------------
void Lcd_box (char Lx1, char Ly1, char Lx2, char Ly2, char Colour)
{
	char X,Y;
	int Pixel,Count;

   	Lcd_window(Lx1,Ly1,Lx2,Ly2);       		//define window
   	X = Lx2 - Lx1;
   	X++;                          		//# of X-Pixel
   	Y = Ly2 - Ly1;
   	Y++;                          		//# of Y-Pixel
   	Pixel = X * Y;                		//# of total pixel
   	for (Count=0;Count<Pixel;Count++)
   	{
		Lcd_sendbyte(PARAM, Colour);	//plot each pixel in the defined output window (=box)
	}
}

//-------------------------------------------------------------------------------
//Clear screen
//-------------------------------------------------------------------------------
void Lcd_clear (void)
{
	Lcd_box(0, 0, 131, 131, WHITE);
}


//-------------------------------------------------------------------------------
//Send Commands or parameter or data byte
//-------------------------------------------------------------------------------
void Lcd_sendbyte (char Command, char Lcdout)
{
	char i;

	LCD_PORT &= ~((1 << CS) | (1 << SCLK));			//Cs 0 start of sequence //Clock 0

    if (Command == CMD)
    {
		LCD_PORT &= ~(1 << SDATA);		//0 = cmd
	}
    else
    {
		LCD_PORT |= (1 << SDATA);		//1 = Parameter
	}

	LCD_PORT |= (1 << SCLK);

   	for (i=0;i<8;i++)
   	{
		LCD_PORT &= ~(1 << SCLK);		//Clock 0

		if (Lcdout & 0x80)
			LCD_PORT |= (1 << SDATA);	//Output MSB

		else
			LCD_PORT &= ~(1 << SDATA);

		LCD_PORT |= (1 << SCLK);		//Clock 1
		Lcdout = Lcdout << 1;			//Shift Data to left

	}
   	LCD_PORT |= (1 << CS);        		//Cs to 1 at end of sequence
}


//-------------------------------------------------------------------------------
//Initializing of the display
//-------------------------------------------------------------------------------
void Lcd_init (void)
{
	char i;

	LCD_TRIS &= ~((1<<RS) | (1<<CS) | (1<<SDATA) | (1<<SCLK);	//Configure I/O

   	delay_ms(10);				//Always wait after a Reset
   	LCD_PORT |= ((1<<RS) | (1<<CS) | (1<<SDATA) | (1<<SCLK);		//Startup LCD

	Lcd_sendbyte(CMD, 0x01);
	delay_ms(10);				//Wait after Reset

    Lcd_sendbyte(CMD, 0x11);	//Sleep Out
	delay_ms(40);

    Lcd_sendbyte(CMD, 0x29);	//Display On
	delay_ms(40);

	Lcd_sendbyte(CMD, 0x3A);	//Interface Pixel Format....

	Lcd_sendbyte(PARAM, 0x02);	//....data format: 8 bit for each Pixel

	Lcd_sendbyte(CMD, 0x20);	//No inversion, 0x21 would be inverted display (different colors!)

	Lcd_sendbyte(CMD, 0x2D);	//RGB-Table for the 256 out of 4096 color mode

	for (i=0;i<20;i++)
	{
		Lcd_sendbyte(PARAM, DataInit[i]);	//read predefined color table for 256 color mode
	}
}


//-------------------------------------------------------------------------------
//Routine for displaying characters
//-------------------------------------------------------------------------------
void Lcd_print (char* String, char Lcd_posx, char Lcd_posy, char Font_size, char Lcd_fcolour, char Lcd_bcolour)
{

	//char Matrix[36] = {	0x7F,0x02,0x04,0x02,0x7F,	//M
						//0x20,0x54,0x54,0x54,0x78,	//a
						//0x04,0x3E,0x44,0x24,0x00,	//t
						//0x44,0x78,0x44,0x04,0x08,	//r
						//0x00,0x00,0x7D,0x00,0x00,	//i
						//0x6C,0x10,0x10,0x6C,0x00};	//x
	char temp[5];

	char X1,Y1,X2,Y2;
	char i,j,k,height,width,pos_Str,count,length_Str;

	char Fontwidth = 1;    	//First we assume small font
	char Fontheight = 1;
	char Xpixel = 5;		//Set Width and height of font
    char Ypixel = 7;

	length_Str = strlen(String);	//Get length of string

    if (Font_size == 1)		//Double Height Sizes
    {
		Fontheight = 2;
		Ypixel = 15;
		height = 2;
	}
	if (Font_size == 2)		//Double Height and Width Sizes
	{
		Fontwidth = 2;
		Fontheight = 2;
		Xpixel = 11;
      	Ypixel = 15;
      	height = 2;
      	width = 2;
	}

	X1 = Lcd_posx;
   	X1 = X1 * 6;             //Each character is 6 pixel wide (incl. one empty collumn at the end)
   	X1 = X1 * Fontwidth;     //starting position, needs to be doubles if using double wide fonts
   	X2 = X1 + (Xpixel - Fontwidth);    //Lcd_window is 5 or 10 pixel wide

   	Y1 = Lcd_posy;
   	Y1 = Y1 * 8;             //each character has a height of 8 pixel
   	Y1 = Y1 * Fontheight;    //needs to be doubles when using double height fonts
   	//Y1++;
   	Y2 = Y1 + (Ypixel);		 //Lcd_window is 8 or 16 pixel tall

	for (i=0;i<length_Str;i++)	//Start at beginning of string and finish at end
	{

		pos_Str = String[i] - 32;	//Calculate place in ASCII memory

		for (count = 0; count < 5; count++)	//Use correct buffer location
		{
			if (pos_Str < 12)
			{
				temp[count]=cASCII1[(pos_Str*5)+count];
			}
			else if (pos_Str < 24)
			{
				temp[count]=cASCII2[((pos_Str - 12)*5)+count];
			}
			else if (pos_Str < 36)
			{
				temp[count]=cASCII3[((pos_Str - 24)*5)+count];
			}
			else if (pos_Str < 48)
			{
				temp[count]=cASCII4[((pos_Str - 36)*5)+count];
			}
			else if (pos_Str < 60)
			{
				temp[count]=cASCII5[((pos_Str - 48)*5)+count];
			}
			else if (pos_Str < 72)
			{
				temp[count]=cASCII6[((pos_Str - 60)*5)+count];
			}
			else if (pos_Str < 84)
			{
				temp[count]=cASCII7[((pos_Str - 72)*5)+count];
			}
			else if (pos_Str < 95)
			{
				temp[count]=cASCII8[((pos_Str - 84)*5)+count];
			}
		}


		Lcd_window (X1+(i*(Xpixel+1)), Y1, X2+(i*(Xpixel+1)), Y2);  //creates single char output window

		for (j=0;j<8;j++)	//For 8 data bits in bytes 0 - 7
		{
			height = 1;
			if (Font_size > 0)
			{
				height = 2;
			}
			while (height > 0)		//If Height Doubled then send twice the amount of y pixels
			{
				for (k=0;k<5;k++)	//For 5 ASCII bytes 0 - 4
				{
					width = 1;
					if (Font_size == 2)
					{
						width = 2;
					}

					while (width > 0) //If Width Doubled then send twice the amount of x pixels
					{
						if (test_bit(temp[k],j))
						{
							Lcd_sendbyte(PARAM, Lcd_fcolour); //Set pixel to foreground colour
						}
						else
						{
							Lcd_sendbyte(PARAM, Lcd_bcolour); //Set pixel to background colour
						}
						width--;
					}
				}
				height--;
			}
		}
	}
}


//---------------------------------------------------------------------------
//Outputs one pixel onto the screen
//---------------------------------------------------------------------------
void Lcd_plot (char X1, char Y1, char Colour)
{
   	Lcd_window(X1,Y1,X1,Y1);		//Creates output window of 1 pixel
   	Lcd_sendbyte(PARAM, Colour);	//set color of pixel
}


//---------------------------------------------------------------------------
//Draws a line from one point to another
//---------------------------------------------------------------------------
void Lcd_drawline (char X1, char Y1, char X2, char Y2, char Colour)
{

   	int C1, M1;
   	signed int D1 = 0;
   	signed int Pixelx = X2 - X1;	//number of X-pixels;
   	signed int Pixely = Y2 - Y1;	//Number of Y-pixels
   	signed char Xinc = 1;
   	signed char Yinc = 1;

	if (Pixelx < 0)		//If negative X direction
	{
		Xinc = -1;
		Pixelx = Pixelx * -1;
	}
	if (Pixely < 0)		//If negative Y direction
	{
		Yinc = -1;
		Pixely = Pixely * -1;
	}
	if (Pixely <= Pixelx)
	{
		C1 = 2 * Pixelx;
		M1 = 2 * Pixely;
		while (X1 != X2)
		{
			Lcd_plot(X1,Y1,Colour);	//Plot line
			X1 = X1 + Xinc;
			D1 = D1 + M1;
			if (D1 > Pixelx)
			{
				Y1 = Y1 + Yinc;
				D1 = D1 - C1;
			}
		}
	}
	else
	{
	  	C1 = 2 * Pixely;
	  	M1 = 2 * Pixelx;
	  	while (Y1 != Y2)
		{
			Lcd_plot(X1,Y1,Colour);	//Plot Line
		 	Y1 = Y1 + Yinc;
		 	D1 = D1 + M1;
		 	if (D1 > Pixely)
		 	{
				X1 = X1 + Xinc;
				D1 = D1 - C1;
			}
		}
		Lcd_plot(X1,Y1,Colour);
	}
}



//---------------------------------------------------------------------------
//Draws a small 16X16 image
//---------------------------------------------------------------------------
void Lcd_drawimage (char X1, char Y1)
{
	char i;
	Lcd_window(X1,Y1,X1+16,Y1+16);

	for (i=0;i<255;i++)
	{
		Lcd_sendbyte(PARAM, i);
	}

}
*/


